from . import app, audit, backup, helpers, security
from .configs import Config


def main(args) -> int:
    if not hasattr(args, 'which'):
        print(
            helpers.bcolors.FAIL
            + 'Can not parse action use --help'
            + helpers.bcolors.ENDC
        )
        return 1

    if hasattr(args, 'debug') and args.debug:
        print(args)

    config = Config(working_dir=args.working_dir)

    if args.which == 'security':
        return security.cli_main(args, config)
    elif args.which == 'audit':
        return audit.cli_main(args, config)
    elif args.which == 'backup':
        return backup.cli_main(args, config)
    elif args.which == 'app':
        return app.cli_main(args, config)

    return 0



import shutil
import tempfile


def cli_main(args, config) -> int:
    if args.action == 'code':
        dirpath = tempfile.mkdtemp()

        print(f'cd {dirpath}')
        print('git clone https://github.com/Talk-Point/semgrep-rules')
        print('git clone <repo>')
        print('cd <repo>')
        print('pip install semgrep')
        print('semgrep --config ../semgrep-rules/')
        print('semgrep --config auto')
        print('npm install -g observatory-cli')
        print('observatory <hostname> --format=report')

        if not args.quit:
            input("Ready for cleanup? (y/n) ")

        shutil.rmtree(dirpath)
    elif args.action == 'app':
        print('observatory <hostname> --format=report')
    elif args.action == 'cloud':
        dirpath = tempfile.mkdtemp()

        print('pip3 install scoutsuite')
        print('gcloud config set project shopcloud-worktable')
        print('scout gcp -u')

        if not args.quit:
            input("Ready for cleanup? (y/n) ")

        shutil.rmtree(dirpath)

    return 0



from pathlib import Path
from typing import List

import yaml

from . import exceptions


class Config:
    FILENAME = 'microservices.yaml'
    VERSION = 'V1'

    def __init__(self, **kwargs):
        self.working_dir = kwargs.get('working_dir', None)
        if self.working_dir == '.':
            self.working_dir = None

    def dict(self) -> dict:
        return {}

    def load_projects(self) -> List[str]:
        filename = 'projects.yaml'

        if self.working_dir is not None:
            filename = Path(self.working_dir) / filename

        if not Path(filename).exists():
            raise exceptions.ProjectFileNotFound()

        projects = []

        with open(filename) as f:
            data = yaml.safe_load(f.read())
            projects = data.get('projects', [])

        return projects



import argparse
import sys

from . import cli

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='Microservice management',
        prog='shopcloud-microservice'
    )

    subparsers = parser.add_subparsers(help='commands', title='commands')
    parser.add_argument('--debug', '-d', help='Debug', action='store_true')
    parser.add_argument('--quit', '-q', help='Quiet', action='store_true')
    parser.add_argument('--simulate', '-s', help='Simulate the process', action='store_true')
    parser.add_argument('--secrethub-token', help='Secrethub-Token', type=str)
    parser.add_argument('--working-dir', help='Working Directory', type=str, default='.')

    parser_security = subparsers.add_parser('security', help='security')
    parser_security.add_argument(
        'action',
        const='generate',
        nargs='?',
        choices=['merge-security-pull-requests', 'ci-secrets-rolling']
    )
    parser_security.add_argument('repo', const='generate', nargs='?')
    parser_security.set_defaults(which='security')

    parser_audit = subparsers.add_parser('audit', help='audit')
    parser_audit.add_argument(
        'action',
        const='generate',
        nargs='?',
        choices=['code', 'cloud', 'app']
    )
    parser_audit.add_argument('repo', const='generate', nargs='?')
    parser_audit.set_defaults(which='audit')

    backup = subparsers.add_parser('backup', help='backup')
    backup.add_argument(
        'action',
        const='generate',
        nargs='?',
        choices=['sql-init', 'sql-list-instances', 'sql-list-databases', 'sql-download', "sql-push-to-drive", "sql-dump"]

    )
    backup.add_argument('name', const='generate', nargs='?')
    backup.set_defaults(which='backup')

    app = subparsers.add_parser('app', help='app')
    app.add_argument(
        'action',
        const='generate',
        nargs='?',
        choices=['health', 'list', 'release']

    )
    app.add_argument('name', const='generate', nargs='?')
    app.set_defaults(which='app')

    args = parser.parse_args()
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    rc = cli.main(args)
    if rc != 0:
        sys.exit(rc)
